// src/index.tsx
import axios from "axios";
import { useEffect, useRef, useState } from "react";

// src/utils/api.ts
var get_base_url = (lang, word) => `https://www.google.com/inputtools/request?ime=transliteration_en_${lang}&num=5&cp=0&cs=0&ie=utf-8&oe=utf-8&app=jsapi&text=${word}`;

// src/utils/helpers.ts
import getCaretCoordinates from "textarea-caret";
var ActionKeys = {
  KEY_ENTER: "Enter",
  KEY_SPACE: " ",
  KEY_TAB: "Tab"
};
var rtlLanguages = ["ur", "ks", "sd"];
function getInputSelection(el) {
  const start = 0;
  const end = 0;
  if (!el) {
    return { start, end };
  }
  if (typeof el.selectionStart === "number" && typeof el.selectionEnd === "number") {
    return { start: el.selectionStart, end: el.selectionEnd };
  }
  return { start, end };
}
var calculateRows = (textLength) => {
  const approximateRows = Math.ceil(textLength / 40);
  return Math.max(1, approximateRows);
};
var updateText = (textRef, setMatchStart, setMatchEnd, setCaretPosition, setLastWord) => {
  if (textRef && textRef.current) {
    const caret = getInputSelection(textRef.current).end;
    const indexOfLastSpace = textRef.current.value.lastIndexOf(" ", caret - 1) < textRef.current.value.lastIndexOf("\n", caret - 1) ? textRef.current.value.lastIndexOf("\n", caret - 1) : textRef.current.value.lastIndexOf(" ", caret - 1);
    setMatchStart(indexOfLastSpace + 1);
    setMatchEnd(caret - 1);
    const caretPos2 = getCaretCoordinates(textRef.current, caret);
    const rect = textRef.current.getBoundingClientRect();
    const left = Math.min(
      caretPos2.left,
      rect.width - 100 / 2
    );
    const top = Math.min(caretPos2.top + 10, rect.height);
    setCaretPosition((prev) => ({ ...prev, top, left }));
    const currentWord = textRef.current.value.slice(indexOfLastSpace + 1, caret);
    if (currentWord === "") {
      return;
    }
    setLastWord(currentWord);
  }
};

// src/utils/styles.ts
var main = {
  width: "100%",
  height: "100%",
  position: "relative"
};
var pretextAreaStyle = {
  width: "100%",
  height: "100%",
  resize: "none",
  textAlign: "start",
  outline: "none"
};
var suggestionsDiv = (caretPosition, offsetX, offsetY) => {
  return {
    backgroundClip: "padding-box",
    backgroundColor: "#fff",
    border: "1px solid rgba(0, 0, 0, 0.15)",
    boxShadow: "0 6px 12px rgba(0, 0, 0, 0.175)",
    display: "block",
    fontSize: "14px",
    listStyle: "none",
    borderRadius: "8px",
    padding: "1px",
    textAlign: "left",
    zIndex: 2e4,
    left: `${caretPosition.left + offsetX}px`,
    top: `${caretPosition.top + offsetY}px`,
    position: "absolute",
    width: "auto"
  };
};
var suggestionsItem = {
  width: "auto",
  height: "auto",
  borderRadius: "8px",
  padding: "2px 8px"
};

// src/index.tsx
import { jsx, jsxs } from "react/jsx-runtime";
function ReactTransliteration({ text = "", onChangeText, language = "hi", keys = [" ", "Enter", "Tab"], enabled = false, renderKey, textRef, placeholder = "", textAreaStyle = {}, suggestionDivStyle = {}, suggestionItemStyle = {}, activeItemStyle = {}, offsetX = 0, offsetY = 0, mainDivStyle = {} }) {
  const [lastWord, setLastWord] = useState("");
  const [selection, setSelection] = useState(0);
  const [matchStart, setMatchStart] = useState(-1);
  const [matchEnd, setMatchEnd] = useState(-1);
  const [options, setOptions] = useState([]);
  const [paletVisible, setPaletVisible] = useState(false);
  const [caretPosition, setCaretPosition] = useState({
    top: 0,
    left: 0
  });
  const timeRef = useRef(null);
  const KEY_UP = "ArrowUp";
  const KEY_DOWN = "ArrowDown";
  const KEY_ESCAPE = "Escape";
  useEffect(() => {
    if (textRef && textRef.current) {
      const rows = calculateRows(text.length);
      textRef.current.rows = rows;
    }
  }, [textRef, text]);
  useEffect(() => {
    if (options.length > 0) {
      handleSelection(0);
      setLastWord("");
    }
  }, [options]);
  const getTransliteration = async () => {
    try {
      const res = await axios.get(get_base_url(language, lastWord));
      if (res.data[1][0][1]?.length > 0) {
        setOptions([...res.data[1][0][1], lastWord]);
      } else {
        setOptions([lastWord]);
      }
    } catch (error) {
      setOptions([lastWord]);
    }
  };
  const handleSelection = (index) => {
    if (textRef && textRef.current) {
      const currentString = textRef.current.value;
      const newValue = currentString.substring(0, matchStart) + options[index] + " " + currentString.substring(matchEnd + 1, currentString.length);
      if (newValue.includes("undefined")) {
        return;
      }
      onChangeText(newValue);
      textRef.current.value = newValue;
      textRef.current.selectionEnd = matchStart + options[index].length + 1;
      if (timeRef.current) {
        clearTimeout(timeRef.current);
      }
      setSelection(0);
    }
  };
  const handleKeyDown = (event) => {
    if (!enabled) {
      return;
    }
    if (keys.includes(event.key)) {
      if (lastWord !== "") {
        if (paletVisible) {
          event.preventDefault();
          handleSelection(selection);
          setPaletVisible(false);
        } else {
          event.preventDefault();
          getTransliteration();
        }
      }
    } else {
      if (textRef && textRef.current) {
        switch (event.key) {
          case KEY_ESCAPE:
            event.preventDefault();
            break;
          case KEY_UP:
            event.preventDefault();
            setSelection((options?.length + selection - 1) % options?.length);
            break;
          case KEY_DOWN:
            event.preventDefault();
            setSelection((selection + 1) % options?.length);
            break;
          case "Backspace":
            if (matchStart + options[selection]?.length + 1 === textRef?.current?.selectionEnd) {
              setPaletVisible(true);
            } else {
              setOptions([]);
              setPaletVisible(false);
            }
            break;
          default:
            break;
        }
      }
    }
  };
  const update = () => {
    if (textRef && textRef.current) {
      onChangeText(textRef?.current?.value);
    }
  };
  const handleMobileSelection = (event, customEvent) => {
    setPaletVisible(false);
    if (customEvent.inputType === "deleteContentBackward") {
      if (matchStart + options[selection]?.length === textRef?.current?.selectionEnd) {
        setPaletVisible(true);
      } else {
        setOptions([]);
        setPaletVisible(false);
        if (event.currentTarget.value === "") {
          setPaletVisible(false);
        }
      }
    }
    if (timeRef.current) {
      clearTimeout(timeRef.current);
    }
    timeRef.current = setTimeout(() => {
      update();
    }, 500);
  };
  return /* @__PURE__ */ jsxs("div", { style: { ...main, ...mainDivStyle }, children: [
    /* @__PURE__ */ jsx(
      "div",
      {
        style: { ...suggestionsDiv(caretPosition, offsetX, offsetY), display: paletVisible ? "block" : "none", ...suggestionDivStyle },
        children: options.length > 0 && options?.map(
          (el, i) => /* @__PURE__ */ jsx(
            "div",
            {
              onClick: () => {
                handleSelection(i);
                setSelection(i);
              },
              style: i !== selection ? { ...suggestionsItem, ...suggestionItemStyle } : { ...suggestionsItem, ...suggestionItemStyle, backgroundColor: "#9e3df2", color: "white", ...activeItemStyle },
              children: el
            }
          )
        )
      }
    ),
    /* @__PURE__ */ jsx(
      "textarea",
      {
        placeholder,
        dir: rtlLanguages.includes(language) ? "rtl" : "ltr",
        ref: textRef,
        onKeyDown: handleKeyDown,
        style: { ...pretextAreaStyle, ...textAreaStyle },
        defaultValue: text,
        onInput: (event) => {
          const customEvent = event.nativeEvent;
          updateText(textRef, setMatchStart, setMatchEnd, setCaretPosition, setLastWord);
          if (window.screen.width < window.screen.height) {
            handleMobileSelection(event, customEvent);
            return;
          }
          if (customEvent.data !== null) {
            setPaletVisible(false);
          }
          if (event.currentTarget.value === "") {
            setPaletVisible(false);
          }
          if (timeRef.current) {
            clearTimeout(timeRef.current);
          }
          timeRef.current = setTimeout(() => {
            update();
          }, 500);
        }
      },
      renderKey ? renderKey : ""
    )
  ] });
}
export {
  ActionKeys,
  ReactTransliteration
};
